"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectionsTypeMap = exports.fixedSizeUint8Array = exports.fixedSizeBuffer = exports.fixedSizeArray = exports.fixedSizeUtf8String = void 0;
const types_1 = require("./types");
const assert_1 = require("assert");
const numbers_1 = require("./numbers");
const package_json_1 = require("../package.json");
const BEET_PACKAGE = package_json_1.name;
const fixedSizeUtf8String = (stringByteLength) => {
    return {
        write: function (buf, offset, value) {
            const stringBuf = Buffer.from(value, 'utf8');
            assert_1.strict.equal(stringBuf.byteLength, stringByteLength, `${value} has invalid byte size`);
            numbers_1.u32.write(buf, offset, stringByteLength);
            stringBuf.copy(buf, offset + 4, 0, stringByteLength);
        },
        read: function (buf, offset) {
            const size = numbers_1.u32.read(buf, offset);
            assert_1.strict.equal(size, stringByteLength, `invalid byte size`);
            const stringSlice = buf.slice(offset + 4, offset + 4 + stringByteLength);
            return stringSlice.toString('utf8');
        },
        byteSize: 4 + stringByteLength,
        description: `utf8-string(${stringByteLength})`,
    };
};
exports.fixedSizeUtf8String = fixedSizeUtf8String;
function fixedSizeArray(element, len, lenPrefix = false) {
    const arraySize = element.byteSize * len;
    const byteSize = lenPrefix ? 4 + arraySize : arraySize;
    return {
        write: function (buf, offset, value) {
            assert_1.strict.equal(value.length, len, `array length ${value.length} should match len ${len}`);
            if (lenPrefix) {
                numbers_1.u32.write(buf, offset, len);
                offset += 4;
            }
            for (let i = 0; i < len; i++) {
                element.write(buf, offset + i * element.byteSize, value[i]);
            }
        },
        read: function (buf, offset) {
            if (lenPrefix) {
                const size = numbers_1.u32.read(buf, offset);
                assert_1.strict.equal(size, len, 'invalid byte size');
                offset += 4;
            }
            const arr = new Array(len);
            for (let i = 0; i < len; i++) {
                arr[i] = element.read(buf, offset + i * element.byteSize);
            }
            return arr;
        },
        byteSize,
        description: `Array<${element.description}>(${len})`,
    };
}
exports.fixedSizeArray = fixedSizeArray;
function fixedSizeBuffer(bytes) {
    return {
        write: function (buf, offset, value) {
            value.copy(buf, offset, 0, bytes);
        },
        read: function (buf, offset) {
            return buf.slice(offset, offset + bytes);
        },
        byteSize: bytes,
        description: `Buffer(len)`,
    };
}
exports.fixedSizeBuffer = fixedSizeBuffer;
function fixedSizeUint8Array(len) {
    const arrayBufferBeet = fixedSizeBuffer(len);
    return {
        write: function (buf, offset, value) {
            const valueBuf = Buffer.from(value);
            arrayBufferBeet.write(buf, offset, valueBuf);
        },
        read: function (buf, offset) {
            const arrayBuffer = arrayBufferBeet.read(buf, offset);
            return Uint8Array.from(arrayBuffer);
        },
        byteSize: len,
        description: `Uint8Array(len)`,
    };
}
exports.fixedSizeUint8Array = fixedSizeUint8Array;
// prettier-ignore
exports.collectionsTypeMap = {
    string: { beet: 'fixedSizeUtf8String', sourcePack: BEET_PACKAGE, ts: 'string', arg: types_1.BEET_TYPE_ARG_LEN },
    Array: { beet: 'fixedSizeArray', sourcePack: BEET_PACKAGE, ts: 'Array', arg: types_1.BEET_TYPE_ARG_LEN },
    Buffer: { beet: 'fixedSizeBuffer', sourcePack: BEET_PACKAGE, ts: 'Buffer', arg: types_1.BEET_TYPE_ARG_LEN },
    Uint8Array: { beet: 'fixedSizeUint8Array', sourcePack: BEET_PACKAGE, ts: 'Uint8Array', arg: types_1.BEET_TYPE_ARG_LEN }
};
//# sourceMappingURL=collections.js.map